@@ -546,8 +546,7 @@ def getContext(kwargs, frame_depth=1):
         raise ValueError("Unknown keyword argument '%s'" % d.popitem()[0])
     if context['truediv'] == 'auto':
         caller_globals = sys._getframe(frame_depth + 1).f_globals
-        context['truediv'] = \
-            caller_globals.get('division', None) == __future__.division
+        context['truediv'] = caller_globals.get('division', None) == __future__.division
 
     return context
 
@@ -624,8 +623,7 @@ def NumExpr(ex, signature=(), **kwargs):
     # translated to either True or False).
 
     context = getContext(kwargs, frame_depth=1)
-    threeAddrProgram, inputsig, tempsig, constants, input_names = \
-        precompile(ex, signature, context)
+    threeAddrProgram, inputsig, tempsig, constants, input_names = precompile(ex, signature, context)
     program = compileThreeAddrForm(threeAddrProgram)
     return interpreter.NumExpr(inputsig.encode('ascii'),
                                tempsig.encode('ascii'),
@@ -726,18 +724,34 @@ def getExprNames(text, context):
 def getArguments(names, local_dict=None, global_dict=None):
     """Get the arguments based on the names."""
     call_frame = sys._getframe(2)
+
+    clear_local_dict = False
     if local_dict is None:
         local_dict = call_frame.f_locals
-    if global_dict is None:
-        global_dict = call_frame.f_globals
+        clear_local_dict = True
+    try:
+        frame_globals = call_frame.f_globals
+        if global_dict is None:
+            global_dict = frame_globals
+
+        # If `call_frame` is the top frame of the interpreter we can't clear its 
+        # `local_dict`, because it is actually the `global_dict`.
+        clear_local_dict = clear_local_dict and not frame_globals is local_dict
+
+        arguments = []
+        for name in names:
+            try:
+                a = local_dict[name]
+            except KeyError:
+                a = global_dict[name]
+            arguments.append(numpy.asarray(a))
+    finally:
+        # If we generated local_dict via an explicit reference to f_locals,
+        # clear the dict to prevent creating extra ref counts in the caller's scope
+        # See https://github.com/pydata/numexpr/issues/310
+        if clear_local_dict:
+            local_dict.clear()
 
-    arguments = []
-    for name in names:
-        try:
-            a = local_dict[name]
-        except KeyError:
-            a = global_dict[name]
-        arguments.append(numpy.asarray(a))
     return arguments
 
 
@@ -812,8 +826,7 @@ def evaluate(ex, local_dict=None, global_dict=None,
     try:
         compiled_ex = _numexpr_cache[numexpr_key]
     except KeyError:
-        compiled_ex = _numexpr_cache[numexpr_key] = \
-                      NumExpr(ex, signature, **context)
+        compiled_ex = _numexpr_cache[numexpr_key] = NumExpr(ex, signature, **context)
     kwargs = {'out': out, 'order': order, 'casting': casting,
               'ex_uses_vml': ex_uses_vml}
     _numexpr_last = dict(ex=compiled_ex, argnames=names, kwargs=kwargs)
